/// <reference types="bn.js" />
import { BN, BorshAccountsCoder, Idl } from "@project-serum/anchor";
import { PublicKey } from "@solana/web3.js";
import { KycProgram } from "../artifacts/kyc-program";
import { KycProgramV1 } from "../artifacts/kyc-program-v1";
export type KycProgramType = KycProgram;
export type KycProgramV1Type = KycProgramV1;
export declare const KycProgramIDL: Idl;
export declare const KycProgramIDL_V1: Idl;
export declare const accountsCoderV1: BorshAccountsCoder<string>;
export declare const accountsCoder: BorshAccountsCoder<string>;
export declare enum AccountName {
    ProviderConfig = "providerConfig",
    UserConfig = "userConfig",
    UserKyc = "userKyc"
}
export declare enum Gender {
    Male = "male",
    Female = "female"
}
export type ProviderConfigData = {
    provider: PublicKey;
    deactivate: boolean;
    admin: PublicKey;
    bump: number[];
};
export type UserConfigData = {
    admin: PublicKey;
    user: PublicKey;
    salt: string;
    kycLevel: number;
    provider: PublicKey;
    deactivate: boolean;
    latestKycAccount: PublicKey;
    bump: number[];
};
export type UserKycData = {
    userConfigAccount: PublicKey;
    name: number[];
    documentId: number[];
    country: string;
    phone: number[];
    dob: number[];
    doe: number[];
    gender: number[];
    isExpired: boolean;
    timestamp: BN;
    prevKycAccount: PublicKey;
    bump: number[];
    provider: string;
};
export type UserKycV1Data = {
    userConfigAccount: PublicKey;
    name: number[];
    documentId: number[];
    country: string;
    dob: number[];
    doe: number[];
    gender: number[];
    isExpired: boolean;
    timestamp: BN;
    prevKycAccount: PublicKey;
    bump: number[];
    provider: string;
};
export type ValidateKycData = {
    encodedName: number[];
    encodedDocumentId: number[];
    encodedCountry: string;
    encodedPhone: number[];
    encodedDob: number[];
    encodedDoe: number[];
    encodedGender: number[];
};
export type IsKycData = {
    isKyc: boolean;
    kycLevel: number | null;
    isExpired: boolean | null;
};
