"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateAndHashKycParams = exports.stringToBytes32 = exports.normalizeName = exports.keypairFromJson = void 0;
const anchor = __importStar(require("@project-serum/anchor"));
const types_1 = require("../types");
const countryLookup = __importStar(require("country-code-lookup"));
const { createHash } = require("crypto");
function keypairFromJson(secretKey) {
    return anchor.web3.Keypair.fromSecretKey(Uint8Array.from(secretKey));
}
exports.keypairFromJson = keypairFromJson;
function normalizeName(name) {
    const removeDiacriticsAndConvertSpecialCharacters = (str) => {
        return str
            .normalize("NFD") // Remove diacritics
            .replace(/[\u0300-\u036f]/g, "") // Convert special characters
            .replace(/Đ/g, "D"); // Replace Đ with D
    };
    const capitalizeFirstLetter = (str) => {
        return str.charAt(0).toUpperCase() + str.slice(1);
    };
    const trimAndSingleSpace = (str) => {
        return str.trim().replace(/\s+/g, " ");
    };
    let normalizedString = removeDiacriticsAndConvertSpecialCharacters(name);
    normalizedString = normalizedString.toLowerCase().trim();
    normalizedString = normalizedString
        .split(" ")
        .map((word) => capitalizeFirstLetter(word))
        .join(" ");
    normalizedString = trimAndSingleSpace(normalizedString);
    return normalizedString;
}
exports.normalizeName = normalizeName;
function hashNTimes(string, salt, n) {
    if (!string || string === "")
        return null;
    let hash = createHash("sha256");
    let result = hash.update(string).update(salt).digest("hex");
    for (let i = 1; i < n; i++) {
        let hashInterval = createHash("sha256");
        hashInterval.update(result);
        result = hashInterval.digest("hex");
    }
    return result;
}
const stringToBytes32 = (str) => {
    if (!str)
        return [];
    const encoder = new TextEncoder();
    const encodedString = encoder.encode(str);
    const bytes32 = new Uint8Array(32);
    bytes32.set(encodedString.subarray(0, 32));
    const numbersArray = Array.from(bytes32, (byte) => byte);
    return numbersArray;
};
exports.stringToBytes32 = stringToBytes32;
const validateAndHashKycParams = (client, kycLevel, salt, name, documentId, country, phone, dob, doe, gender) => {
    // Validate salt
    if (!salt || salt === "") {
        throw Error("Salt is required!");
    }
    if (salt.length > 20) {
        throw Error("Invalid salt length! Cannot exceed 20 characters");
    }
    // Validate phone number
    if (!phone || phone === "") {
        throw Error("Phone number is required!");
    }
    if (phone.length > 255) {
        throw Error("Invalid phone length! Cannot exceed 255 characters");
    }
    // Require parameters for kyc level higher than 1
    if (kycLevel !== 1) {
        if (!name || name === "") {
            throw Error("Name is required!");
        }
        ;
        if (!documentId || documentId === "") {
            throw Error("Document ID is required!");
        }
        ;
        if (!country || country === "") {
            throw Error("Country is required!");
        }
        ;
    }
    let encodedName = null;
    if (name) {
        if (name.length > 255) {
            throw Error("Invalid name length! Cannot exceed 255 characters");
        }
        encodedName = normalizeName(name);
    }
    if (documentId) {
        if (documentId.length > 255) {
            throw Error("Invalid document ID length! Cannot exceed 255 characters");
        }
    }
    if (phone) {
        if (phone.length > 255) {
            throw Error("Invalid phone number length! Cannot exceed 255 characters");
        }
    }
    if (country.length !== 2) {
        throw Error("Invalid country! Please provider valid country code");
    }
    let encodedCountry = country.toLowerCase();
    if (encodedCountry !== "eu" && encodedCountry !== "go" && !countryLookup.byIso(encodedCountry)) {
        throw Error("Invalid country");
    }
    let encodedGender = [];
    if (gender) {
        if (!Object.values(types_1.Gender).includes(gender)) {
            throw Error("Invalid gender");
        }
        encodedGender = (0, exports.stringToBytes32)(hashNTimes(gender, salt, client.genderHashRound));
    }
    return {
        encodedName: encodedName
            ? (0, exports.stringToBytes32)(hashNTimes(encodedName, salt, client.nameHashRound))
            : [],
        encodedDocumentId: documentId
            ? (0, exports.stringToBytes32)(hashNTimes(documentId, salt, client.documentIdHashRound))
            : [],
        encodedCountry,
        encodedPhone: (0, exports.stringToBytes32)(hashNTimes(phone, salt, client.phoneHashRound)),
        encodedDob: dob
            ? (0, exports.stringToBytes32)(hashNTimes(dob.getTime().toString(), salt, client.dobHashRound))
            : [],
        encodedDoe: doe
            ? (0, exports.stringToBytes32)(hashNTimes(doe.getTime().toString(), salt, client.doeHashRound))
            : [],
        encodedGender,
    };
};
exports.validateAndHashKycParams = validateAndHashKycParams;
