import { Instruction } from "@orca-so/common-sdk";
import { PublicKey, SystemProgram, SYSVAR_RENT_PUBKEY } from "@solana/web3.js";
import { Program } from "@project-serum/anchor";
import { KycProgram } from "../artifacts/kyc-program";
import { KycProgramV1 } from "../artifacts/kyc-program-v1";

export type UpdateKycParams = {
  accounts: {
    authority: PublicKey;
    providerConfigAccount: PublicKey;
    userConfigAccount: PublicKey;
    oldUserKycAccount: PublicKey;
    newUserKycAccount: PublicKey;
  };
  inputs: {
    kycLevel: number;
    name: number[];
    documentId: number[];
    country: string;
    phone: number[];
    dob: number[];
    doe: number[];
    gender: number[];
    isExpired: boolean;
  };
};

export async function updateKyc(
  program: Program<KycProgram>,
  program_v1: Program<KycProgramV1>,
  params: UpdateKycParams,
): Promise<Instruction> {
  const { accounts, inputs } = params;

  let ix;
  try {
    await program.methods
      .updateKyc(
        inputs.kycLevel,
        inputs.name,
        inputs.documentId,
        inputs.country,
        inputs.phone,
        inputs.dob,
        inputs.doe,
        inputs.gender,
        inputs.isExpired,
      )
      .accounts({
        ...accounts,
        systemProgram: SystemProgram.programId,
        rent: SYSVAR_RENT_PUBKEY,
      })
      .simulate();
    ix = await program.methods
      .updateKyc(
        inputs.kycLevel,
        inputs.name,
        inputs.documentId,
        inputs.country,
        inputs.phone,
        inputs.dob,
        inputs.doe,
        inputs.gender,
        inputs.isExpired,
      )
      .accounts({
        ...accounts,
        systemProgram: SystemProgram.programId,
        rent: SYSVAR_RENT_PUBKEY,
      })
      .instruction();
  } catch(_err) {
    console.log("Cannot create instruction with new program, retry with old version");
    ix = await program_v1.methods
      .updateKyc(
        inputs.name,
        inputs.documentId,
        inputs.country,
        inputs.dob,
        inputs.doe,
        inputs.gender,
        inputs.isExpired,
      )
      .accounts({
        ...accounts,
        systemProgram: SystemProgram.programId,
        rent: SYSVAR_RENT_PUBKEY,
      })
      .simulate();
    ix = await program_v1.methods
      .updateKyc(
        inputs.name,
        inputs.documentId,
        inputs.country,
        inputs.dob,
        inputs.doe,
        inputs.gender,
        inputs.isExpired,
      )
      .accounts({
        ...accounts,
        systemProgram: SystemProgram.programId,
        rent: SYSVAR_RENT_PUBKEY,
      })
      .instruction();
  }

  return {
    instructions: [ix],
    cleanupInstructions: [],
    signers: [],
  };
}
